#include "main.h"
#include "CH224K.h"
#include "bsp_delay.h"
#include "bsp_iwdg.h"

#include "MonoScreen.h"

#define VOLTAGE_FACTOR       1.0F                 // ʵʵѹԵѹıֵ
#define CURRENT_FACTOR       1.0F                 // ʵʵԵıֵ

#define VOLTAGE_ADC_CHANNEL  ADC_Channel_02_PA1   // ѹͨ
#define CURRENT_ADC_CHANNEL  ADC_Channel_03_PA2   // ͨ

void BSP_ADC_Init(void);
uint16_t BSP_ADC_GetData(uint8_t ADC_Channel);

int main(void)
{
	// ڵʱÿŹ
	DBG_Peripheral_ON(DBG_IWDG_STOP);

	// ʼ
	MonoScreen_Init();
	BSP_ADC_Init();
	CH224K_Init();

	// ʾȫ0.5루ʾ
	if (RCC_Flag_Status_Get(RCC_FLAG_IWDGRST) == RESET)
	{
		MonoScreen_FillScreen(0xFF);
		MonoScreen_Flush();
		SysTick_Delay_Ms(500);
		MonoScreen_FillScreen(0x0);
		MonoScreen_Flush();

		RCC_Reset_Flag_Clear();
	}

	// ÿŹ
	IWDG_Config(IWDG_CONFIG_PRESCALER_DIV32, 0xFF0);

	// ADC1LSBʵʵѹıֵ
	const float volFactor = 8.471446586200685F * VOLTAGE_FACTOR;
	const float curFactor = 1.646382291543582F * CURRENT_FACTOR;
	// ѹ͵ADCֵ
	uint32_t volRaw = 0, curRaw = 0;
	// ѹ͵ֵλmV/mA
	uint16_t voltage = 0, current = 0;
	// 幦ֵλ
	uint32_t power = 0;

	char strbuf[256];
	while (1)
	{
		PD_9V();
		// ѹ͵ADCֵ16
		volRaw = 0;
		curRaw = 0;
		for (int i = 0; i < 16; i++)
		{
			volRaw += BSP_ADC_GetData(VOLTAGE_ADC_CHANNEL);
			curRaw += BSP_ADC_GetData(CURRENT_ADC_CHANNEL);
			SysTick_Delay_Ms(20);
		}
		volRaw >>= 4;
		curRaw >>= 4;

		// ʵʵĵѹ͹
		voltage = volRaw * volFactor;
		current = curRaw * curFactor;
		power = current * voltage / 1000;

		// ʾʾ
		MonoScreen_ClearScreen();
		MonoScreen_setFontSize(2, 2);

		sprintf(strbuf, "%u.%02uV", voltage / 1000, (voltage % 1000) / 10);
		MonoScreen_DrawString(0, 1, strbuf);

		sprintf(strbuf, "%u.%02uA", current / 1000, (current % 1000) / 10);
		MonoScreen_DrawString(0, 17, strbuf);

		// ҪжʾСλ
		if (power <= 9999)
		{
			sprintf(strbuf, "%u.%02uW", power / 1000, (power % 1000) / 10);
		}
		else
		{
			sprintf(strbuf, "%u.%uW", power / 1000, (power % 1000) / 100);
		}
		MonoScreen_setFontSize(2, 4);
		MonoScreen_DrawString(67, 2, strbuf);

		// ˢĻʾ
		MonoScreen_Flush();
		// here we go:)
		IWDG_Feed();
	}
}

void BSP_ADC_Init()
{
	// ʱ
	RCC_AHB_Peripheral_Clock_Enable(RCC_AHB_PERIPH_GPIOA | RCC_AHB_PERIPH_ADC);
	ADC_Clock_Mode_Config(ADC_CKMOD_AHB, RCC_ADCHCLK_DIV16);
	RCC_ADC_1M_Clock_Config(RCC_ADC1MCLK_SRC_HSI, RCC_ADC1MCLK_DIV8); // selsect HSE as RCC ADC1M CLK Source

	GPIO_InitType GPIO_InitStructure;

	// ADCùģʽΪģģʽ
	GPIO_Structure_Initialize(&GPIO_InitStructure);
	GPIO_InitStructure.Pin = GPIO_PIN_1 | GPIO_PIN_2;
	GPIO_InitStructure.GPIO_Mode = GPIO_MODE_ANALOG;
	GPIO_Peripheral_Initialize(GPIOA, &GPIO_InitStructure);
	
	ADC_InitType ADC_InitStructure;

	// ADC
	ADC_InitStructure.MultiChEn = DISABLE;
	ADC_InitStructure.ContinueConvEn = DISABLE;
	ADC_InitStructure.ExtTrigSelect = ADC_EXT_TRIGCONV_REGULAR_SWSTRRCH;
	ADC_InitStructure.DatAlign = ADC_DAT_ALIGN_R;
	ADC_InitStructure.ChsNumber = ADC_REGULAR_LEN_1;
	ADC_Initializes(&ADC_InitStructure);

	// ADC
	ADC_ON();
	// ȴADC
	while (ADC_Flag_Status_Get(ADC_RD_FLAG, ADC_FLAG_AWDG, ADC_FLAG_RDY) == RESET)
	{
		__NOP();
	}

	// ADCУ׼
	ADC_Calibration_Operation(ADC_CALIBRATION_ENABLE);
	// ȴADCУ׼
	while (ADC_Calibration_Operation(ADC_CALIBRATION_STS) == SET)
	{
		__NOP();
	}
}

uint16_t BSP_ADC_GetData(uint8_t ADC_Channel)
{
	// ò
	ADC_Channel_Sample_Time_Config(ADC_Channel, ADC_SAMP_TIME_55CYCLES5);
	ADC_Regular_Sequence_Conversion_Number_Config(ADC_Channel, ADC_REGULAR_NUMBER_1);

	// ADCת
	ADC_Regular_Channels_Software_Conversion_Operation(ADC_EXTRTRIG_SWSTRRCH_ENABLE);
	// ȴADCת
	while (ADC_Flag_Status_Get(ADC_RUN_FLAG, ADC_FLAG_ENDC, ADC_FLAG_RDY) == 0)
	{
		__NOP();
	}
	ADC_Flag_Status_Clear(ADC_FLAG_ENDC);
	ADC_Flag_Status_Clear(ADC_FLAG_STR);
	
	// ȡADCֵ
	return ADC_Regular_Group_Conversion_Data_Get();
}

